@@ -1,7 +1,7 @@
 /* Android activity for Gforth on Android
 
   Authors: Bernd Paysan, Anton Ertl
-  Copyright (C) 2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023 Free Software Foundation, Inc.
+  Copyright (C) 2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023,2024 Free Software Foundation, Inc.
 
   This file is part of Gforth.
 
@@ -49,12 +49,14 @@ import android.hardware.usb.UsbManager;
 import android.content.Context;
 import android.view.View;
 import android.view.Window;
+import android.view.WindowInsets;
 import android.view.KeyEvent;
 import android.view.MotionEvent;
 import android.view.Surface;
 import android.view.SurfaceView;
 import android.view.SurfaceHolder;
 import android.view.OrientationEventListener;
+import android.view.View.OnLayoutChangeListener;
 import android.view.ViewTreeObserver.OnGlobalLayoutListener;
 import android.view.WindowManager;
 import android.view.inputmethod.InputMethodManager;
@@ -74,6 +76,7 @@ import android.app.PendingIntent;
 import android.app.Notification;
 import android.app.NotificationManager;
 import android.app.NotificationChannel;
+import android.app.slice.SliceManager;
 import android.net.ConnectivityManager;
 import android.net.Uri;
 import android.util.Log;
@@ -89,6 +92,9 @@ import java.io.InputStream;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.util.Locale;
+import androidx.core.content.pm.ShortcutManagerCompat;
+import androidx.core.content.pm.ShortcutInfoCompat;
+import androidx.core.graphics.drawable.IconCompat;
 import gnu.gforth.R;
 
 public class Gforth
@@ -147,8 +153,13 @@ public class Gforth
     public Runnable rsecurescreenoff;
     public Runnable notifyer;
     public Runnable startbrowser;
+    public Runnable addshortcut;
     public ProgressDialog progress;
     public String cameraPath;
+    public String shortcutname;
+    public String shortcutfile;
+    public String shortcuticon;
+    public WindowInsets insets;
 
     private static final String META_DATA_LIB_NAME = "android.app.lib_name";
     private static final String META_DATA_STARTFILE = "android.app.startfile";
@@ -349,6 +360,12 @@ public class Gforth
 	    mActivity.onEventNative(15, h);
 	}
 	@Override
+	public WindowInsets onApplyWindowInsets(WindowInsets insets) {
+	    mActivity.insets=insets;
+	    mActivity.onEventNative(27, insets);
+	    return insets;
+	}
+	@Override
 	public boolean dispatchKeyEvent (KeyEvent event) {
 	    mActivity.onEventNative(0, event);
 	    return true;
@@ -450,12 +467,7 @@ public class Gforth
 	if(mView!=null) mView.restartIME();
     }
     public void showStatus() {
-	if (Build.VERSION.SDK_INT < 16) {
-	    getWindow().clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
-	}
-	else {
-	    getWindow().getDecorView().setSystemUiVisibility(0);
-	}
+	getWindow().getDecorView().setSystemUiVisibility(0);
     }
     public void hideStatus() {
 	// Hide Status Bar
@@ -505,10 +517,24 @@ public class Gforth
 	try {
             ai = getPackageManager().getActivityInfo(intent.getComponent(), PackageManager.GET_META_DATA);
             if (ai.metaData != null) {
+		for (String key : ai.metaData.keySet()) {
+		    Log.v(TAG, "metaData."+key+"="+ai.metaData.getString(key));
+		}
                 String ln = ai.metaData.getString(META_DATA_LIB_NAME);
-                if (ln != null) libname = ln;
+                if (ln != null) {
+		    libname = ln;
+		}
                 String sf = ai.metaData.getString(META_DATA_STARTFILE);
-                if (sf != null) startfile = sf;
+                if (sf != null) {
+		    startfile = sf;
+		}
+		Uri uri = intent.getData();
+		if (uri != null) {
+		    Log.v(TAG, "uri="+uri.toString());
+		    startfile = uri.toString().substring(7);
+		}
+		Log.v(TAG, "libname="+libname);
+		Log.v(TAG, "startfile="+startfile);
             }
         } catch (PackageManager.NameNotFoundException e) {
             throw new RuntimeException("Error getting activity info", e);
@@ -637,6 +663,24 @@ public class Gforth
 		    startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse(args0)));
 		}
 	    };
+	addshortcut=new Runnable() {
+		public void run() {
+		    Context context = getApplicationContext();
+		    if (ShortcutManagerCompat.isRequestPinShortcutSupported(context)) {
+			IconCompat icon = (shortcuticon != null) ?
+			    IconCompat.createWithContentUri("file.//"+shortcuticon) :
+			    IconCompat.createWithResource(context, R.drawable.ic_launcher);
+			ShortcutInfoCompat shortcutInfo = new ShortcutInfoCompat.Builder(context, shortcutfile)
+			    .setIntent(new Intent(context, Gforth.class)
+				       .setAction(Intent.ACTION_MAIN)
+				       .setData(Uri.parse("file://"+shortcutfile)))
+			    .setShortLabel(shortcutname)
+			    .setIcon(icon)
+			    .build();
+			ShortcutManagerCompat.requestPinShortcut(context, shortcutInfo, null);
+		    }
+		}
+	    };
 	
 	recKeepalive = new BroadcastReceiver() {
 		@Override public void onReceive(Context context, Intent foo)
@@ -646,10 +690,12 @@ public class Gforth
 		    onEventNative(21, 0);
 		}
 	    };
-	registerReceiver(recKeepalive, new IntentFilter("gnu.gforth.keepalive") );
-	
-	pintent = PendingIntent.getBroadcast(this, 0, new Intent("gnu.gforth.keepalive"), PendingIntent.FLAG_MUTABLE);
+	if (Build.VERSION.SDK_INT >= 26) {
+	    registerReceiver(recKeepalive, new IntentFilter("gnu.gforth.keepalive"), RECEIVER_NOT_EXPORTED );
+	}
 
+	pintent = PendingIntent.getBroadcast(this, 0, new Intent("gnu.gforth.keepalive"), PendingIntent.FLAG_IMMUTABLE);
+	
 	// intent for notifications
 	gforthintent = PendingIntent.getActivity
 	    (this, 1,
