@@ -1,7 +1,7 @@
 /* common header file
 
   Authors: Bernd Paysan, Anton Ertl, David Kühling, Jens Wilke, Neal Crook
-  Copyright (C) 1995,1996,1997,1998,2000,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023 Free Software Foundation, Inc.
+  Copyright (C) 1995,1996,1997,1998,2000,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023,2024 Free Software Foundation, Inc.
 
   This file is part of Gforth.
 
@@ -391,17 +391,6 @@ typedef Label *Xt;
 #define IF_rpTOS(x)
 #define rpTOS (rp[0])
 
-typedef struct {
-  Cell next_task;
-  Cell prev_task;
-  Cell save_task;
-  Cell* sp0;
-  Cell* rp0;
-  Float* fp0;
-  Address lp0;
-  Xt *throw_entry;
-} user_area;
-
 typedef struct {
   Address base;		/* base address of image (0 if relocatable) */
   UCell dict_size;
@@ -411,12 +400,13 @@ typedef struct {
   Address sect_primbits;
   Address sect_targets;
   Address sect_codestart;
+  Address sect_last_header;
+  Address sect_litstack;
   UCell data_stack_size;
   UCell fp_stack_size;
   UCell return_stack_size;
   UCell locals_stack_size;
   Xt *boot_entry;	/* initial ip for booting (in BOOT) */
-  Xt *throw_entry;	/* ip after signal (in THROW) */
   Xt *quit_entry;
   Xt *execute_entry;
   Xt *find_entry;
@@ -428,6 +418,18 @@ typedef struct {
 } ImageHeader;
 /* the image-header is created in main.fs */
 
+typedef struct {
+  Cell next_task;
+  Cell prev_task;
+  Cell save_task;
+  Cell* sp0;
+  Cell* rp0;
+  Float* fp0;
+  Address lp0;
+  Xt *throw_entry;
+  ImageHeader *current_section;
+} user_area;
+
 typedef struct {
   Address base;
   Cell size;
@@ -553,8 +555,8 @@ Address gforth_alloc(Cell size);
 char *cstr(Char *from, UCell size);
 char *tilde_cstr(Char *from, UCell size);
 Cell opencreate_file(char *s, Cell wfam, int flags, Cell *wiorp);
-DCell timeval2us(struct timeval *tvp);
-DCell timespec2ns(struct timespec *tvp);
+DCell gf_timeval2us(struct timeval *tvp);
+DCell gf_timespec2ns(struct timespec *tvp);
 void cmove(Char *c_from, Char *c_to, UCell u);
 void cmove_up(Char *c_from, Char *c_to, UCell u);
 Cell compare(Char *c_addr1, UCell u1, Char *c_addr2, UCell u2);
@@ -589,6 +591,7 @@ int gf_ungottenc(FILE *stream);
 /* signal handler stuff */
 void install_signal_handlers(void);
 void throw(int code);
+void gforth_fail();
 /* throw codes */
 #define BALL_DIVZERO     -10
 #define BALL_RESULTRANGE -11
@@ -625,7 +628,7 @@ void vm_count_block(Xt *ip);
 
 /* dynamic superinstruction stuff */
 void compile_prim1(Cell *start);
-void gforth_compile_range(Cell *image, UCell size,
+void gforth_compile_range(Cell *image, Cell size,
 			  Char *bitstring, Char *targets);
 void finish_code(void);
 void finish_code_barrier(void);
@@ -637,7 +640,7 @@ extern Cell fetch_decompile_prim(Cell *a_addr);
 int state_map(int);
 
 extern int offset_image;
-extern int die_on_signal;
+extern Cell die_on_signal;
 extern int ignore_async_signals;
 extern UCell pagesize;
 extern Address dictguard;
@@ -691,16 +694,28 @@ extern double asinh(double r1);
 extern double acosh(double r1);
 #endif
 #ifndef HAVE_ECVT
-/* extern char* ecvt(double x, int len, int* exp, int* sign);*/
+#if defined(_AIX) && defined(_NOTHROW)
+extern char *_NOTHROW(ecvt, (double, int, int *, int *));
+#else
+extern char* ecvt(double x, int len, int* exp, int* sign);
+#endif
 #endif
 #ifndef HAVE_MEMMOVE
-/* extern char *memmove(char *dest, const char *src, long n); */
+#if defined(_AIX) && defined(_NOTHROW)
+extern void *_NOTHROW(memmove, (void *, const void *, size_t));
+#else
+extern char *memmove(void *dest, const void *src, size_t n);
+#endif
 #endif
 #ifndef HAVE_SINCOS
 extern void sincos(double x, double *s, double *c);
 #endif
 #ifndef HAVE_ECVT_R
+# ifdef _AIX
+extern int _NOTHROW(ecvt_r, (double x, int ndigits, int* exp, int* sign, char *buf, int len));
+# else
 extern int ecvt_r(double x, int ndigits, int* exp, int* sign, char *buf, size_t len);
+# endif
 #endif
 #ifndef HAVE_POW10
 extern double pow10(double x);
